/*

dllimpl.h - Header for use by DLL implementors

This essentially defines the calling conventions of the entrypoints to the
DLLs. This varys depending upon the operating system involved.

Entrypoints should be written like this example :-

	DLLEXPORT int DLLENTRY my_strlen(const char *s)
		{
		return strlen(s);
		}

Pointers to entrypoints should be manipulated thus :-

	DLL_EP ep_strlen = dll_entrypoint(dll, "my_strlen", "4");
	( int (DLL_EP_PTR ep_strlen)(const char *) )("hello") == 5

*/

#ifndef DLLIMPL_H
#define DLLIMPL_H

#if   defined(OS2)
#define	DLLEXPORT
#define	DLLENTRY _System
typedef void (* DLL_EP)(void);
#ifdef __cplusplus
#define	DLL_EP_PTR _System *
#else
#define	DLL_EP_PTR * _System
#endif

#elif defined(O16)
#define	DLLEXPORT
#define	DLLENTRY _pascal _far _loadds
typedef void (_pascal _far _loadds * DLL_EP)(void);
#define	DLL_EP_PTR _pascal _far _loadds *

#elif defined(WIN32)
#define	DLLEXPORT __declspec(dllexport)
#define	DLLENTRY __stdcall
typedef void (* DLL_EP)(void);
#define	DLL_EP_PTR __stdcall *

#elif defined(DOS32)
#define	DLLEXPORT
#define	DLLENTRY __export _cdecl
typedef void (* DLL_EP)(void);
#define	DLL_EP_PTR _cdecl *

#elif defined(xxDOS)
#define	DLLEXPORT
#define	DLLENTRY _loadds
typedef void (_loadds * DLL_EP)(void);
#define	DLL_EP_PTR _loadds *

#else
#define	DLLEXPORT
#define	DLLENTRY
typedef void (* DLL_EP)(void);
#define	DLL_EP_PTR *
#endif

#if defined(AIX) || defined(xxDOS)
typedef struct { DLL_EP ep; const char *name; } DLL_EXPORT;
extern DLL_EXPORT * __start(void);
#endif

#endif
