/*

dlluser.c - Code to be linked with  DLL users (UNIX version)

The AIX version of this code is slightly different to support C++.
However, I've yet to arrange a C++ compile to take advantage of this.

*/

/*...sincludes:0:*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#if defined(AIX)
#include <dlfcn.h>
#include <sys/ldr.h>
#ifdef __cplusplus
#include <load.h>
#endif
#elif defined(LINUX) || defined(SUN)
#include <dlfcn.h>
#elif defined(HP)
#include <dl.h>
#endif
#include "dllimpl.h"
#include "dlluser.h"

/*...vdllimpl\46\h:0:*/
/*...vdlluser\46\h:0:*/
/*...e*/

#ifdef AIX
typedef struct
	{
  #ifdef AIX_DLOPEN
	void *handle;
  #endif
  #ifdef AIX_LOAD
	DLL_EXPORT * (*fn)(void);
  #endif
	} IDLL;
#endif

DLL_EP dll_entrypoint(DLL *dll, const char *name, const char *args)
	{
#if defined(AIX)
	IDLL *idll = (IDLL *) dll;
	args=args; /* Not relevant on AIX */
  #ifdef AIX_DLOPEN
	if ( idll->handle != NULL )
		return (DLL_EP) dlsym((void *) dll->handle, name);
  #endif
  #ifdef AIX_LOAD
	{
	DLL_EXPORT *e;
	for ( e = idll->fn(); e->ep != (DLL_EP) 0; e++ )
		if ( !strcmp(e->name, name) )
			return e->ep;
	}
  #endif
	return (DLL_EP) NULL;
#elif defined(LINUX) || defined(SUN)
	void *handle = (void *) dll;
	DLL_EP ep;
	args=args; /* Not relevant on Linux or SunOS */
	ep = (DLL_EP) dlsym(handle, name);
	return ( dlerror() == 0 ) ? ep : (DLL_EP) NULL;
#elif defined(HP)
	shl_t handle = (shl_t) dll;
	DLL_EP ep;
	args=args; /* Not relevant on HP-UX */
	return shl_findsym(&handle, name, TYPE_PROCEDURE, &ep) == -1 ?
		(DLL_EP) NULL : ep;
#else
	name=name; args=args; /* Suppress warnings */
	return (DLL_EP) NULL;
#endif
	}

DLL *dll_load(const char *name)
	{
#if defined(AIX)
	IDLL *idll;
	if ( (idll = (IDLL *) malloc(sizeof(IDLL))) == NULL )
		return NULL; 
  #ifdef AIX_DLOPEN
	{
	char dllname[200+1+2+1];
	strcpy(dllname, name);
	strcat(dllname, ".so");
	if ( (idll->handle = dlopen(dllname, RTLD_NOW)) != NULL )
		return (IDLL *) idll;
	}
  #endif
  #ifdef AIX_LOAD
    #ifdef __cpluscplus
	idll->fn = (DLL_EXPORT * (*)(void)) loadAndInit(name, 0, getenv("PATH"));
    #else
	idll->fn = (DLL_EXPORT * (*)(void)) load((char *) name, 0, (char *) getenv("PATH"));
    #endif
	if ( (idll->fn != NULL )
		return (DLL *) idll;
  #endif
	free(idll);
	return (DLL *) NULL;
#elif defined(LINUX) || defined(SUN)
	char dllname[200+1+2+1];
	strcpy(dllname, name);
	strcat(dllname, ".so");
	return (DLL *) dlopen(dllname, RTLD_NOW);
#elif defined(HP)
	strcpy(dllname, name);
	strcat(dllname, ".sl");
	return shl_load(dllname, BIND_DEFERRED|DYNAMIC_PATH, 0L);
#else
	/* This type of UNIX has no DLL support yet */
	name=name; /* Suppress warning */
	return (DLL *) NULL;
#endif
	}

void dll_unload(DLL *dll)
	{
#if defined(AIX)
	IDLL *idll = (IDLL *) dll;
  #ifdef AIX_DLOPEN
	if ( idll->handle != NULL )
		{ dlclose(idll->handle); free(idll); return; }
  #endif
  #ifdef AIX_LOAD
    #ifdef __cplusplus
	terminateAndUnload((int (*)()) fn);
    #else
	unload((void *) fn);
    #endif
  #endif
	free(idll);
#elif defined(LINUX)
	void *handle = (void *) dll;
	dlclose(handle);
#elif defined(HP)
	shl_t handle = (shl_t) dll;
	shl_unload(handle);
#else
	dll=dll; /* Suppress warning */
#endif
	}
