/*

dllimpl.c - Main C code to be linked with DLLs (16 bit DOS version)

To load a DOS DLL called DLLNAME, execute the program DLLNAME.DLL, passing
arguments "DOS-TSR-DLL", "seg_ept" and "off_ept".

The DLL will store to seg_ept:off_ept the address of its entrypoint table.
This table includes the textual name and far call address of each entrypoint,
and is terminated by a entry of two NULL pointers.

Included is an entrypoint by the name of "_dll_init", which must be called by
the DLL loading program as soon as the load is complete. Similarly, the
"_dll_term" entrypoint should be called by the DLL loading program to terminate
and unload the DLL.

*/

/*...sincludes:0:*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <dos.h>
#include "dllimpl.h"

/*...vdllimpl\46\h:0:*/
/*...e*/

extern void _cdecl dllasm_exit(void); /* eg: like _exit(0); */
extern void _cdecl dllasm_deinit(void); /* does not return */

static void DLLENTRY dll_init(void) {}
static void DLLENTRY dll_term(void) { dllasm_deinit(); }

int main(int argc, char *argv[])
	{
	DLL_EXPORT *e, *ept, **p_ept;
	int i, n;

	/* If we haven't got the other arguments, then the odds are the
	   user is trying to run this program as if it were a program.
	   A program loading this code as a DOS-TSR-DLL would get it
	   right, we hope. */
	if ( argc != 4 || strcmp(argv[1], "DOS-TSR-DLL") )
		{
		fputs("This is a DOS-TSR-DLL, not an executable program.\n", stderr);
		return 1;
		}

	/* Where should we write the address of our entrypoint table. */
	FP_SEG(p_ept) = atoi(argv[2]);
	FP_OFF(p_ept) = atoi(argv[3]);

	/* Count entrypoints */
	e = __start();
	for ( n = 0; e[n].name != NULL; n++ )
		;

	if ( (ept = (DLL_EXPORT *) malloc((n+3)*sizeof(DLL_EXPORT))) == NULL )
		{
		*p_ept = (DLL_EXPORT *) 0xffff0002;
		return 2;
		}

	for ( i = 0; i < n; i++ )
		{
		ept[i].name = e[i].name;
		ept[i].ep   = e[i].ep;
		}

	ept[i].name = "_dll_init";
	ept[i].ep   = (DLL_EP) dll_init;
	i++;

	ept[i].name = "_dll_term";
	ept[i].ep   = (DLL_EP) dll_term;
	i++;

	ept[i].name = NULL;
	ept[i].ep   = (DLL_EP) NULL;

	/* Declare location of entrypoint table. */
	*p_ept = ept;

	/* Terminate and stay resident */
	dllasm_exit();
	/* NOT REACHED */

	return 0; /* Keep compiler happy */
	}
